﻿-- Author      : RisM
-- Create Date : 6/28/2009 3:57:34 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("gui/gui.lua")



-------------------------------------------------------------------------------
-- OPTIONS GUI FUNCTIONS - SHARED UTILITIES
-------------------------------------------------------------------------------


function SpeakinSpell:AddToBlizOptions(FrameObjectName, FrameDisplayName, ParentDisplayName)
	-- AddToBlizOptions(appName="SpeakinSpell", display name, parent, ObjectName)
	-- NOTE: the FrameObjectName here represents self.OptionsGUI.args[FrameObjectName]
	self.optionsFrames[FrameObjectName] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SpeakinSpell", FrameDisplayName, ParentDisplayName, FrameObjectName)
	
	-- for some reason the framework has a problem if we define the "reset defaults" button handler in the data before calling AddToBlizOptions
	-- so we have to add it afterwards
	self.optionsFrames[FrameObjectName].default = function() self:ResetToDefaults(FrameObjectName) end
end



function SpeakinSpell:RefreshFrame(FrameName)
	-- the only way I found to do this is to hide and then show the frame
	-- this occurs fast enough that I do not see a flicker when this happens
	-- UPDATE: saw a forum post when I was at work with the proper API call, but I already forget what it was, and this still works
	if self.optionsFrames and self.optionsFrames[FrameName] and self.optionsFrames[FrameName]:IsVisible() then
		self.optionsFrames[FrameName]:Hide()
		self.optionsFrames[FrameName]:Show()
	end
end



function SpeakinSpell:GUI_GetEventSelectLabel()
	local type = SpeakinSpell.RuntimeData.OptionsGUIStates.SelectedEventTypeFilter
	if type == "MACRO" then
		return L.EVENTTYPES.IN_SPELL_LIST[type]..L["macro"]
	end
	local name = L.EVENTTYPES.IN_SPELL_LIST[type]
	if name then
		return name
	else
		return L["Select a Spell"]
	end
end


--FormatDisplayName
-- create a display name for an event, using options
-- typonly = a type filter in effect, or nil
--		if a type filter is in effect, don't include the type in the display name
--		if a type filter is NOT in effect, include the event type in the display name
--
function SpeakinSpell:FormatDisplayName( de, DisplayNameFormat )
	local funcname = "FormatDisplayName"
	
	local DefaultFormat = {
		typeonly = "ALL", 
		ShowAllRanks = true, 
		HighlightFilterText = false, 
		BaseColor = "|r",
	}
	local Format = self:CopyTable( DisplayNameFormat )
	self:ValidateObject( Format, DefaultFormat )
		
	-- rank = " (Rank)" or ""
	local rank = ""
	if self:IsEventTypeRankable( de.type ) then
		rank = de.rank
		if de.rank == "" then
			if Format.ShowAllRanks then
				rank = L[" (Any Rank)"]
			else
				rank = ""
			end
		else
			rank = string.format( L[" (%s)"], de.rank )
		end
	else
		rank = ""
	end
		
	local ColorMatchedName = ""
	if Format.HighlightFilterText then
		ColorMatchedName = self:ColorFilterText( de.name, Format.BaseColor )
	else
		ColorMatchedName = (Format.BaseColor)..de.name
	end
	
	
	if Format.typeonly == "ALL" then -- show the type in the display name
		return (Format.BaseColor)..L.EVENTTYPES.IN_SPELL_LIST[ de.type ].. ColorMatchedName..rank
	end
	
	-- when showing macros only, format so we show...
	-- when I type: /ss macro
	-- foo, bar, battlecry, whatever
	if (Format.typeonly == "MACRO") and (de.type == "MACRO") then
		if self:StartsWith( string.lower( de.name ), L["macro"] ) then
			ColorMatchedName = string.sub( de.name, string.len( L["macro"] )+1 )
			if Format.HighlightFilterText then
				ColorMatchedName = self:ColorFilterText( ColorMatchedName, Format.BaseColor )
			end
			return (Format.BaseColor)..ColorMatchedName
		end
	end
	
	-- For all other types, we're only showing this type of event, so show only the name and rank
	return ColorMatchedName..rank
end


-------------------------------------------------------------------------------
-- FILTER EVENT LIST SEARCH FUNCTION
-------------------------------------------------------------------------------

	
function SpeakinSpell:RebuildSpellList( values, OptionsGUIState, EventTable, MatchesFilterFunc, GetDisplayNameFunc )
	local funcname = "RebuildSpellList"
	
	if not OptionsGUIState.FilterChanged then
		--self:DebugMsg(funcname,"not changed")
		return
	end
	
	--self:DebugMsg(funcname,"entry")
	OptionsGUIState.FilterChanged = false

	-- if the current selection doesn't match the filter, erase it, and we'll pick a new one
	if OptionsGUIState.SelectedEventKey and not MatchesFilterFunc( OptionsGUIState.SelectedEventKey ) then
		--self:DebugMsg(funcname,"erasing selection")
		OptionsGUIState.SelectedEventKey = nil
	end
	
	-- search NewEventsDetected for those that match the filter
	-- add or remove from the list as needed
	numAdded = 0
	for key,_ in pairs(EventTable) do
		-- if we're allowed to add more
		-- and this event matches the filter
		if (OptionsGUIState.SelectedEventKey == key) or -- we already matched the selected key
			(SpeakinSpellSavedData.ShowMoreThanAHundred or (numAdded < 100)) and MatchesFilterFunc(key) then
			-- add to the list of shown values
			numAdded = numAdded + 1
			values[key] = GetDisplayNameFunc(key)
			-- make sure we have a current spell selected for the options GUI 
			-- we'll use the first one we see
			if not OptionsGUIState.SelectedEventKey then
				--self:DebugMsg(funcname,"set selection to:"..key)
				OptionsGUIState.SelectedEventKey = key
			end
		else
			-- remove it from the list of shown values
			-- NOTE: keep the current selection, even if that makes 101 items
			values[key] = nil
		end
	end
	
	--table.sort(values) -- appears to be self-sorting
end


-------------------------------------------------------------------------------
-- ShowPage() functions - Open the Interface Options Panel to various pages
-------------------------------------------------------------------------------


function SpeakinSpell:ShowOptions()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory( "SpeakinSpell" )
end


function SpeakinSpell:ShowOptions_Toggle()
	local funcname = "ShowOptions_Toggle"
	-- open interface options to show SpeakinSpell
	-- if it's already open, close it
	local FrameName = SpeakinSpellSavedData.LastPageViewed.ObjectName --L["General"]
	self:DebugMsg(funcname, "FrameName:"..FrameName)
	if self.optionsFrames and self.optionsFrames[FrameName] and self.optionsFrames[FrameName]:IsVisible() then
		self.optionsFrames[FrameName]:GetParent():GetParent():Hide()
	else
		self:ShowLastPageViewed()
	end
end


function SpeakinSpell:SetLastPageViewed( ObjectName, InterfacePanel )
	local funcname = "SetLastPageViewed"
	--self:DebugMsg(funcname, "InterfacePanel:"..InterfacePanel )
	
	SpeakinSpellSavedData.LastPageViewed = {
		ObjectName = ObjectName,
		InterfacePanel = InterfacePanel,
	}
end


function SpeakinSpell:ShowLastPageViewed()
	local funcname = "ShowLastPageViewed"
	local InterfacePanel = SpeakinSpellSavedData.LastPageViewed.InterfacePanel
	--self:DebugMsg(funcname, "InterfacePanel:"..InterfacePanel)
	InterfaceOptionsFrame_OpenToCategory( InterfacePanel )
end


function SpeakinSpell:ShowHelp()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["SpeakinSpell Help"])
end


function SpeakinSpell:ShowImport()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Import New Data"])
end


function SpeakinSpell:ShowCreateNew()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Create New..."])
end


function SpeakinSpell:ShowMessageOptions()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Message Settings TITLE"])
end


function SpeakinSpell:ShowRandomSubsOptions()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Random Substutitions"])
end


-------------------------------------------------------------------------------
-- CreateGUI - add the options GUI to the game's interface panel
-------------------------------------------------------------------------------


function SpeakinSpell:CreateGUI()
	-- Define Options table for the GUI
	self.OptionsGUI.name = "SpeakinSpell"
	
	-- add dynamically-generated controls to the GUI definitions
	self:CreateGUI_CurrentMessagesGUI()
	self:CreateGUI_RandomSubs()
	self:CreateGUI_HelpPages()
	self:CreateGUI_Import()
	
	-- add the options to the in-game interface options panel at Escape > Interface > Addons > SpeakinSpell
	-- The ordering here matters, it determines the order in the Blizzard Interface Options
	-- (or it would matter if we had sub-categories, which we currently don't)
	-- InterfaceOptions_AddCategory(self.OptionsGUI) -- done via Ace below

	-- :RegisterOptionsTable(appName, options, slashcmd, persist)
	-- AceConfig:RegisterOptionsTable("MyAddon", myOptions, {"/myslash", "/my"})
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SpeakinSpell", self.OptionsGUI, nil)
	
	--SpeakinSpell:AddToBlizOptions(FrameObjectName, FrameDisplayName, ParentDisplayName)
	--frame objects are stored to self.optionsFrames[FrameObjectName]
	self.optionsFrames = {}
	--TODO: make these redundant lists data-driven ... iterate on SpeakinSpell.OptionsGUI.args or something
	self:AddToBlizOptions("General",		"SpeakinSpell",		nil)
	self:AddToBlizOptions("CurrentMessagesGUI",	L["Message Settings TITLE"],"SpeakinSpell")
	self:AddToBlizOptions("CreateNew",		L["Create New..."],				"SpeakinSpell")
	self:AddToBlizOptions("RandomSubs",		L["Random Substutitions"],		"SpeakinSpell")
	--self:AddToBlizOptions("ImportGUI",		L["Import New Data"],			"SpeakinSpell")
	--self:AddToBlizOptions("Trade",		L["Trade"],						"SpeakinSpell") --TODOFUTURE: implement a trade barker feature
	self:AddToBlizOptions("HelpRoot",		L["SpeakinSpell Help"],			"SpeakinSpell")
	
	-- add the minimap button
	self:MinimapButton_Create()
end


-- if the options GUI is showing right now, we need to refresh it to make it show the new data
function SpeakinSpell:RefreshAllFrames()
	self:RefreshFrame("General")
	self:RefreshFrame("CurrentMessagesGUI")
	self:RefreshFrame("CreateNew")
	self:RefreshFrame("RandomSubs")
	--self:RefreshFrame("ImportGUI")
	--self:RefreshFrame("Trade")
end

