﻿-- Author      : RisM
-- Create Date : 6/28/2009 3:38:58 PM

-------------------------------------------------------------------------------
-- Create Addon Object
-------------------------------------------------------------------------------

SpeakinSpell = LibStub("AceAddon-3.0"):NewAddon("SpeakinSpell", "AceConsole-3.0", "AceEvent-3.0")

-------------------------------------------------------------------------------
-- CURRENT VERSION
-------------------------------------------------------------------------------

SpeakinSpell.CURRENT_VERSION = "3.2.2.04"

-------------------------------------------------------------------------------
-- DEBUG vs. RETAIL
-------------------------------------------------------------------------------

SpeakinSpell.DEBUG_MODE = false

-------------------------------------------------------------------------------
-- CONSTANTS
-------------------------------------------------------------------------------

SpeakinSpell.URL = "http://www.wowace.com/projects/speakinspell/"

SpeakinSpell.ALL_URLs = [[
http://wow.curse.com/downloads/wow-addons/details/speakinspell.aspx
http://www.wowace.com/projects/speakinspell/
http://www.wowace.com/addons/speakinspell/tickets/
]]

SpeakinSpell.MAX = {
	MESSAGES_PER_SPELL		= 100, -- do not ever make this number smaller, but we can always make it bigger
	TEXTS_PER_RANDOM_SUB	= 100, -- also do not shrink
}


-------------------------------------------------------------------------------
-- COLORS
-------------------------------------------------------------------------------

SpeakinSpell.Colors = {
	Channels = {
		BATTLEGROUND= "|cffff6b00",
		RAID		= "|cffff6b00",
		SPEAKINSPELL= "|cff33ff99",
		GUILD		= "|c4444ff44",
		PARTY		= "|cffaaeeff",
		EMOTE		= "|cffff7b34",
		YELL		= "|cffff2f32",
		RAID_WARNING= "|cffffdbad",
		SELFRAIDWARN= "|cffffdbad",
	},
	SearchMatch		= "|cffff2222",
	ON				= "|cff00ff00",
	OFF				= "|cffff0000",
	ClickHighlight	= "|cffffff00",
	Substitutions	= "|cffff2222",
	SELFRAIDWARNFrame = { -- raid warning color ff-db-ad defined as floating point numbers
		r = (255/255), --ff
		g = (219/255), --db
		b = (173/255), --ad
	},
}


-------------------------------------------------------------------------------
-- DEFAULTS
-------------------------------------------------------------------------------


SpeakinSpell.DEFAULTS = {
	FREQUENCY = 1.00, -- means 100%
	COOLDOWN = 0, -- default of 0 seconds = no cooldown
	
	AD_CHANNELS = { -- for /ss ad
		Arena		= "PARTY",
		BG			= "SAY",
		WG			= "SAY",
		RaidLeader	= "RAID",
		RaidOfficer	= "RAID",
		Raid		= "RAID",
		Party		= "PARTY",
		PartyLeader	= "PARTY",
		Solo		= "SAY",
	},
	
	-- Saved Data Format and Defaults
	SpeakinSpellSavedData = {
		-- Data Format Version/Schema
		Version = SpeakinSpell.CURRENT_VERSION,
		
		-- General Settings
		ShowSpellEvents = false,
		ShowDebugMessages = false,
		EnableAllMessages = true,
		ShowAllRanks = false,
		ShowMoreThanAHundred = false,
		ShowMinimapButton = true,
		ShowVersionAtLogin = true,
		ShowImportProgress = false,
		
		-- Last Page Viewed
		-- page opened with "/ss toggle" or minimap button
		LastPageViewed = {
			ObjectName = "General",
			InterfacePanel = "SpeakinSpell",
		},
		
		-- Data Tables
		EventTable = {},
		RandomSubs = {},
		NewEventsDetected = {},
	},
	
	SpeakinSpellSavedDataForAll = {
		-- Data Format Version/Schema
		Version = SpeakinSpell.CURRENT_VERSION,
		-- Data Tables
		SpellIdCache = {}
	},
	
	DetectedEventStub = {
		-- all stubs are expected to have at least a name and type, which MUST exist
		name = "UNKNOWN",
		type = "EVENT",
		rank = "",
		
		-- the rest is optional, including...
		--spellid = 1234,
		--target = "Stonarius",
		--caster = "Stonarius",
	},
	
	DetectedEvent = {
		-- everything from a DetectedEventStub, plus...
		
		-- caster/target info is set by the original stub or CreateDetectedEvent
		--caster = "Stonarius",
		--target = "Stonarius",
		
		-- table keys -- generated in Validate_DetectedEvent / MakeEventTableKeys
		--key = "TABLEKEY",
		--rankedkey = "TABLEKEYRANK",
		--ranklesskey = "TABLEKEY",
	},
	
	-- Speech Events
	EventTableEntry = {
		DetectedEventStub = {-- describes the event to announce
			name = "UNKNOWN",
			type = "EVENT",
		}, -- describes the event to announce
		
		-- General settings for this event
		DisableAnnouncements = false,
		
		-- The speech list
		Messages = {}, -- populated with default sample speeches from DEFAULT_SPEECHES in template.lua OnInitialize
		
		-- Channel options / Whisper
		WhisperTarget = false,
		Channels = { -- for creating new spells
			Solo		= "SPEAKINSPELL CHANNEL",
			Party		= nil, --"SAY",
			Raid		= nil,
			PartyLeader	= nil, --"SAY",
			RaidLeader	= nil, --"SAY",
			RaidOfficer	= nil, --"SAY",
			BG			= nil,
			Arena		= nil,
			WG			= nil,
		},
		
		-- Spam Reduction options
		Frequency = 1.00,
		Cooldown  = 0,
	},
	
	Templates = {}, -- Working copy of DEFAULT_SPEECHES.Templates
}


-------------------------------------------------------------------------------
-- DECLARE OPTIONS GUI TABLE
-------------------------------------------------------------------------------
SpeakinSpell.OptionsGUI = {
	type = "group",
	args = {},
}

-------------------------------------------------------------------------------
-- SAVED DATA DECLARATION
-------------------------------------------------------------------------------

-- our main saved data is stored in SpeakinSpellSavedData.EventTable
-- we must declare the parent table SpeakinSpellSavedData
-- but we know whether real data has been loaded or not if SpeakinSpellSavedData.EventTable exists
-- NOTE: SpeakinSpellSavedData.EventTable may be an empty table
SpeakinSpellSavedData = {}
SpeakinSpellSavedDataForAll = {}

-------------------------------------------------------------------------------
-- BASIC PRINT FUNCTIONS
-------------------------------------------------------------------------------


function SpeakinSpell:Print(message)
	-- Overriding AceConsole-3.0:Print to use localized addon name instead of tostring(self)
	local text = SpeakinSpell.Colors.Channels.SPEAKINSPELL.."SpeakinSpell|r: " .. message
	DEFAULT_CHAT_FRAME:AddMessage( text )
end


function SpeakinSpell:PrintLoading(filename)
	--self:Print("loaded "..filename) -- used for debugging purposes
end

SpeakinSpell:PrintLoading("loader.lua")
